#ifndef Basic2DVector_H
#define Basic2DVector_H

#include <cmath>
// ROOT addition
#include "TObject.h"
   
//using namespace std;

template < class T> 
class Basic2DVector {

public:
  // default constructor
  Basic2DVector() : theX(0), theY(0) {}

  // copy constructor from same type
  Basic2DVector( const Basic2DVector & p) : 
    theX(p.x()), theY(p.y()) {}

#ifndef CMS_NO_TEMPLATE_MEMBERS
  //  // copy constructor and implicit conversion from other point classes 
  //  template <class OtherPoint> 
  //  Basic2DVector( const OtherPoint& p) : 
  //    theX(p.x()), theY(p.y()) {}
#endif 

  Basic2DVector( const T& x, const T& y) : 
    theX(x), theY(y) {}

  T x() const { return theX;}
  T y() const { return theY;}
  T mag2() const { return theX*theX + theY*theY;}
  T r() const    { return sqrt( mag2());}
  T mag() const  { return r();}
  T phi() const  { return theX == 0.0 && theY == 0.0 ? 0.0 : atan2(theY,theX);}

  Basic2DVector& operator+= ( const Basic2DVector& p) {
    theX += p.x();
    theY += p.y();
    return *this;
  } 

  Basic2DVector& operator-= ( const Basic2DVector& p) {
    theX -= p.x();
    theY -= p.y();
    return *this;
  } 

  Basic2DVector operator-() const { return Basic2DVector(-x(),-y());}

  Basic2DVector& operator*= ( const T& t) {
    theX *= t;
    theY *= t;
    return *this;
  }

  T dot( const Basic2DVector& v) const { return x()*v.x() + y()*v.y();}

private:
  T theX;
  T theY;
  
// ROOT addition
  ClassDefT(Basic2DVector,1)
};

// ROOT addition
  ClassDefT2(Basic2DVector,T)

     
template<class T> 
Basic2DVector<T> operator+( const Basic2DVector<T>& v1, 
			    const Basic2DVector<T>& v2) {
  return Basic2DVector<T>(v1) += v2;
}

template<class T> 
Basic2DVector<T> operator-( const Basic2DVector<T>& v1, 
			    const Basic2DVector<T>& v2) {
  return Basic2DVector<T>(v1) -= v2;
}

// scalar product
template <class T>
T operator*( const Basic2DVector<T>& v1, const Basic2DVector<T>& v2) {
  return v1.dot(v2);
}

template <class T>
Basic2DVector<T> operator*( const Basic2DVector<T>& v, const T& t) {
  return Basic2DVector<T>(v) *= t;
}

template <class T>
Basic2DVector<T> operator*( const T& t, const Basic2DVector<T>& v) {
  return v*t;
}

#endif // Basic2DVector_H
