#ifndef PV2DBase_H
#define PV2DBase_H

#include "Basic2DVector.h"
#include <iosfwd>
// ROOT addition
#include "TObject.h"

template <class T, class PVType, class FrameType>
class PV2DBase {
public:

  typedef T                ScalarType;
  typedef Basic2DVector<T> BasicVectorType;

  PV2DBase() : theVector() {}

  PV2DBase( const T& x, const T& y) : theVector(x,y) {}

  PV2DBase( const Basic2DVector<T>& v) : theVector(v) {}

  const BasicVectorType& basicVector() const { return theVector;}

  T x() const { return basicVector().x();}
  T y() const { return basicVector().y();}
  T mag2() const { return basicVector().mag2();}
  T r() const    { return basicVector().r();}
  T mag() const  { return basicVector().mag();}
  T phi() const  { return basicVector().phi();}

  // PV2DBase& operator+=( const Global2DVector<T>& v) {
  //   theVector += v.basicVector();
  //   return *this;
  // }

protected:
  BasicVectorType& basicVector() { return theVector;}

private:
  BasicVectorType theVector;

// ROOT addition
  ClassDefT(PV2DBase,1)
};

// ROOT addition
  ClassDef3T2(PV2DBase,T,PVType,FrameType)

template <class T, class PV, class F>
ostream & operator<<( ostream& s, const PV2DBase<T,PV,F>& v) {
  return s << " (" << v.x() << ',' << v.y() << ") ";
}
#endif // PV2DBase_H
