#ifndef PV3DBase_H
#define PV3DBase_H

#include "Basic3DVector.h"
#include <iosfwd>
// ROOT addition
#include "TObject.h"

template <class T, class PVType, class FrameType>
class PV3DBase {
public:
  typedef Basic3DVector<T> BasicVectorType;

  PV3DBase() : theVector() {}

  PV3DBase(const T & x, const T & y, const T & z) : theVector(x, y, z) {}

  PV3DBase( const Basic3DVector<T>& v) : theVector(v) {}

  const BasicVectorType& basicVector() const { return theVector;}
  
  T x() const { return basicVector().x();}
  T y() const { return basicVector().y();}
  T mag2() const { return basicVector().mag2();}
  T r() const    { return basicVector().r();}
  T mag() const  { return basicVector().mag();}
  T phi() const  { return basicVector().phi();}

  T z() const     { return basicVector().z();}
  T perp2() const { return basicVector().perp2();}
  T perp() const  { return basicVector().perp();}
  T transverse() const  { return basicVector().transverse();}
  T theta() const { return basicVector().theta();}
  T eta() const   { return basicVector().eta();}

  //  PV3DBase& operator+=( const Global3DVector<T>& v) {
  //    theVector += v.basicVector();
  //    return *this;
  //} 
    
protected:
  BasicVectorType& basicVector() { return theVector;}

private:
  BasicVectorType theVector;
  
// ROOT addition
  ClassDefT(PV3DBase,1)
};

// ROOT addition
  ClassDef3T2(PV3DBase,T,PVType,FrameType)


template <class T, class PV, class F>
ostream & operator<<( ostream& s, const PV3DBase<T,PV,F>& v) {
  return s << " (" << v.x() << ',' << v.y() << ',' << v.z() << ") ";
} 

#endif // PV3DBase_H
