#ifndef Point2DBase_H
#define Point2DBase_H


#include "PointTag.h"
#include "PV2DBase.h"
#include "Vector2DBase.h"
// ROOT addition
#include "TObject.h"

template <class T, class FrameTag>
class Point2DBase : public PV2DBase< T, PointTag, FrameTag> {
public:

  typedef PV2DBase< T, PointTag, FrameTag>    BaseClass;
  typedef Vector2DBase< T, FrameTag>          VectorType;
  typedef Basic2DVector<T>                    BasicVectorType;

  Point2DBase() {}

  Point2DBase(const T& x, const T& y) : BaseClass(x, y) {}

  explicit Point2DBase( const BasicVectorType& v) : BaseClass(v) {}

  Point2DBase& operator+=( const VectorType& v) {
    basicVector() += v.basicVector();
    return *this;
  } 

  Point2DBase& operator-=( const VectorType& v) {
    basicVector() -= v.basicVector();
    return *this;
  } 
  T theZ;
  
// ROOT addition
  ClassDefT(Point2DBase,1)

};

// ROOT addition
  ClassDef2T2(Point2DBase,T,FrameTag)


template<class T, class F>
Point2DBase<T,F> operator+( const Point2DBase<T,F>& p,
			    const Point2DBase<T,F>::VectorType& v) {
  return Point2DBase<T,F>(p) += v;
}

template<class T, class F>
Point2DBase<T,F> operator+( const Point2DBase<T,F>::VectorType& v,
			    const Point2DBase<T,F>& p) {
  return p + v;
}

template<class T, class F>
Point2DBase<T,F>::VectorType operator-( const Point2DBase<T,F>& p1,
					const Point2DBase<T,F>& p2) {
  return Point2DBase<T,F>::VectorType( p1.basicVector() - p2.basicVector());
}

#endif // Point2DBase_H
