#ifndef Vector2DBase_H
#define Vector2DBase_H


#include "VectorTag.h"
#include "PV2DBase.h"
// ROOT addition
#include "TObject.h"

template <class T, class FrameTag> 
class Vector2DBase : public PV2DBase<T, VectorTag, FrameTag> {
public:

  typedef PV2DBase<T, VectorTag, FrameTag>    BaseClass;

  Vector2DBase() {}

  Vector2DBase(const T& x, const T& y) : BaseClass(x, y) {}

  explicit Vector2DBase( const Basic2DVector<T>& v) : BaseClass(v) {}

  Vector2DBase& operator+= ( const Vector2DBase& v) {
    basicVector() += v.basicVector();
    return *this;
  } 
    
  Vector2DBase& operator-= ( const Vector2DBase& v) {
    basicVector() -= v.basicVector();
    return *this;
  } 
    
  Vector2DBase operator-() const { return Vector2DBase(-basicVector());}

  Vector2DBase& operator*= ( const T& t) { 
    basicVector() *= t;
    return *this;
  } 

  T dot( const Vector2DBase& v) const { 
    return basicVector().dot( v.basicVector());;
  }
  
// ROOT addition
  ClassDefT(Vector2DBase,1)

};

// ROOT addition
  ClassDef2T2(Vector2DBase,T,FrameTag)

   
template <class T, class FrameTag>
Vector2DBase<T,FrameTag> operator+( const Vector2DBase<T,FrameTag>& v1, 
				    const Vector2DBase<T,FrameTag>& v2) {
  return Vector2DBase<T,FrameTag>(v1) += v2;
}

template <class T, class FrameTag>
Vector2DBase<T,FrameTag> operator-( const Vector2DBase<T,FrameTag>& v1, 
				    const Vector2DBase<T,FrameTag>& v2) {
  return Vector2DBase<T,FrameTag>(v1) -= v2;
}

template <class T, class FrameTag>
T operator*( const Vector2DBase<T,FrameTag>& v1, 
	     const Vector2DBase<T,FrameTag>& v2) {
  return v1.dot(v2);
}

template <class T, class FrameTag>
Vector2DBase<T,FrameTag> operator*( const Vector2DBase<T,FrameTag>& v, 
				    float t) {
  return Vector2DBase<T,FrameTag>(v) *= t;
}

template <class T, class FrameTag>
Vector2DBase<T,FrameTag> operator*( float t, 
				    const Vector2DBase<T,FrameTag>& v) {
  return v*t;
}

template <class T, class FrameTag>
Vector2DBase<T,FrameTag> operator*( const Vector2DBase<T,FrameTag>& v, 
				    double t) {
  return Vector2DBase<T,FrameTag>(v) *= t;
}

template <class T, class FrameTag>
Vector2DBase<T,FrameTag> operator*( double t, 
				    const Vector2DBase<T,FrameTag>& v) {
  return v*t;
}

#endif // Vector2DBase_H
